from random import random, randint
from math import floor, ceil
from statistics import mean,median

from network import Network

class Agent():
	def __init__(self):
		# unelegant: here is the scale of choices defined
		self.pref = randint(1,11)
		self.norm = -1
	
	def choose(self, peers):
		choices = [self.pref]
		for p in peers:
			choices.append(p.norm)
		self.norm = median(choices)
		# median is not necessarily discrete -> discretize at random
		if self.norm % 1 != 0:
			if random() < 0.5:
				self.norm = floor(self.norm)
			else:
				self.norm = ceil(self.norm)

central_influence_switch = True
#assign the desired value of the central influence
central_influence = 8
central_agent = None

dynamic_logging_switch = False
#have to be defined, if dynamic logging is switched on!
choices_filename = "choices.txt"
preferences_filename = "preferences.txt"

choices_file = None
preferences_file = None

def init_network(n_initial):
	global choices_file, preferences_file
	if dynamic_logging_switch:
		choices_file = open(choices_filename, "w")
		preferences_file = open(preferences_filename, "w")
	net = Network()
	for i in range(0, n_initial):
		a = Agent()
		a.choose([])
		net.add_node(a)
	for i in range(0, n_initial):
		for j in range(i, n_initial):
			net.add_edge(i,j)
	return net
	
def run(nagents, m=2, m0=10):
	if central_influence_switch:
		central_agent = Agent()
		central_agent.pref = central_influence
		central_agent.norm = central_influence
	net = init_network(m0)
	agents = []
	for n in net.nodes:
		agents.append(n)
	for t in range(m0, nagents):
		#log the dynamic evolution of the distributions
		if dynamic_logging_switch:
			for a in agents:
				choices_file.write(str(a.norm)+" ")
				preferences_file.write(str(a.pref)+" ")
			choices_file.write("\n")
			preferences_file.write("\n")
		a = Agent()
		net.preferential_attach(a, m)
		neighbors = net.neighbors(a)
		if central_influence_switch:
			#Implement a central source of influence
			neighbors.append(central_agent)
		a.choose(neighbors)
		agents.append(a)
	if dynamic_logging_switch:
		choices_file.close()
		preferences_file.close()
	return agents

if __name__ == "__main__":
	n = 1000
	agents = run(1000, m=5, m0=20)
	from matplotlib import pyplot as plt
	norm_choice = []
	preference = []
	for a in agents:
		norm_choice.append(a.norm)
		preference.append(a.pref)
	print(abs(mean(norm_choice) - mean(preference)))
	plt.hist(norm_choice, bins=11, range=(1,11))
	plt.show()
	plt.hist(preference, bins=11, range=(1,11))
	plt.show()
